/*
Copyright (C) 2007-2009 Ramon Ziai
Copyright (C) 2007-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */


package de.linuxusers.levenshtein.algos;

import de.linuxusers.levenshtein.editweight.EqualsWeightCalculator;
import de.linuxusers.levenshtein.editweight.WeightCalculator;

/**
 * A class encapsulating the Levenshtein algorithm.
 * 
 * @author Ramon Ziai
 * @author Niels Ott
 * @version $Id: LevenshteinDistance.java 42 2010-05-17 12:46:34Z niels $
 * @param <T> the type of which the objects are that are fed into the algorithm.
 */
public class LevenshteinDistance<T> {

	protected WeightCalculator<T> calculator;
	protected float lastDistance;

	/**
	 * Constructs a new object for computing Levenshtein Distance
	 * using the {@link EqualsWeightCalculator} as default.
	 * Note that this behavior changed after version 0.1.0
	 * @since 0.2.0
	 */
	public LevenshteinDistance() {
		this(new EqualsWeightCalculator<T>());
	}

	/**
	 * Constructs a new object for computing Levenshtein Distance
	 * using a specific {@link WeightCalculator}.
	 * @param calculator the specific calculator to compute the distance. 
	 */
	public LevenshteinDistance(WeightCalculator<T> calculator) {
		lastDistance = 0;
		this.calculator = calculator;
	}

	/**
	 * Computes the Levensthein distance between two sequences.
	 * If both sequences are empty, the cost of 
	 * {@link WeightCalculator#getReplaceWeight(Object, Object)} is returned,
	 * which will be 0.0 for any reasonable implementation.
	 * Replacing empty sequences by other sequences yields the cost of inserting
	 * the elements in the other sequence.
	 * Replacing any sequence by an empty sequences yields the cost of deleting
	 * the elements in the initial sequence.
	 * 
	 * @param seq1 the initial sequence
	 * @param seq2 the sequence to transform the initial one into
	 * @return the cost of transforming the first sequence into the latter.
	 */
	public float getDistance(T[] seq1, T[] seq2) {
		int m = seq1.length;
		int n = seq2.length;
		float[][] dist = new float[m+1][n+1];
		
		// special case: both sequences are empty
		if ( seq1.length == 0 && seq2.length == 0) {
			return calculator.getReplaceWeight(null, null);
		}
		// special case: first sequence is empty: cost is the one of inserting the second sequence
		if ( seq1.length == 0 ) {
			float cost = 0;
			for ( int i = 0; i < seq2.length; i++ ) {
				cost += calculator.getInsertWeight(seq2[i]);
			}
			return cost;
		}
		// special case: second sequence is empty: cost is the one of deleting the first sequence
		if ( seq2.length == 0 ) {
			float cost = 0;
			for ( int i = 0; i < seq1.length; i++ ) {
				cost += calculator.getDeleteWeight(seq1[i]);
			}
			return cost;
		}
		
		
		
		// Levenshtein's famous algorithm:
		for (int i = 0; i <= m; i++) {
			for (int j = 0; j <= n; j++) {
				float upper = Float.MAX_VALUE,
				left = Float.MAX_VALUE,
				upperleft = Float.MAX_VALUE;
				
				
				if (i > 0) {
					upper = dist[i-1][j] + calculator.getDeleteWeight(seq1[i-1]);
				}
				
				if (i > 0 && j > 0) {
					upperleft = dist[i-1][j-1] + calculator.getReplaceWeight(seq1[i-1], seq2[j-1]);
				}
				
				if (j > 0) {
					left = dist[i][j-1] + calculator.getInsertWeight(seq2[j-1]);
				}
				
				dist[i][j] = min(upper, upperleft, left);
				if (dist[i][j] == Float.MAX_VALUE) {
					dist[i][j] = 0;
				}
			}
		}
		//printMatrix(dist);
		lastDistance = dist[m][n];
		return lastDistance;
	}
	
	public static void printMatrix(float[][] matrix) {
		for (float[] i : matrix) {
			for (float j : i) {
				System.err.print(j + " ");
			}
			System.err.println();
		}
	}
	
	private float min(float d1, float d2, float d3) {
		float min = d1;
		
		if (d2 < min) {
			min = d2;
		}
		
		if (d3 < min) {
			min = d3;
		}
		
		return min;
	}

	/**
	 * Sets the weight calculator to use for computing the distance.
	 * @param calculator the calculator to use.
	 * @deprecated specify the calculator in the constructor {@link #LevenshteinDistance(WeightCalculator)} instead.
	 */
	public void setWeightCalculator(WeightCalculator<T> calculator) {
		this.calculator = calculator;
	}

	public double getLastDistance() {
		return lastDistance;
	}

}
