/*
Copyright (C) 2007-2009 Ramon Ziai
Copyright (C) 2007-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */


package de.linuxusers.levenshtein.editweight;


/**
 * Generic weight calculator that compares any type of object
 * using {@link #equals(Object)}. 
 * Objects passed to this calculator should provide a 
 * reasonable implementation of {@link #equals(Object)}.
 * 
 * @param <T> the type of object to compare to using this calculator, must implement {@link #equals(Object)}.
 * @version $Id: EqualsWeightCalculator.java 42 2010-05-17 12:46:34Z niels $
 * @author Ramon Ziai
 * @author Niels Ott
 */
public class EqualsWeightCalculator<T> implements WeightCalculator<T> {

	/**
	 * @param del the element that is to be deleted.
	 * @return 0 if the argument is null (deleting nothing is for free), otherwise 1.
	 * @see WeightCalculator#getDeleteWeight(Object)
	 */
	public float getDeleteWeight(T del) {
		if ( del == null ) {
			return 0;
		}
		return 1;
	}

	/**
	 * @param ins the element that is to be inserted
	 * @return 0 if the argument is null (inserting nothing is for free), otherwise 1.
	 * @see WeightCalculator#getInsertWeight(Object)
	 */
	public float getInsertWeight(T ins) {
		if ( ins == null ) {
			return 0;
		}
		return 1;
	}

	/**
	 * Simples inuitive implementation of replacing things: replacing
	 * equal things is for free (weight 0), everything else is not (weight 1).
	 * Replacing null by null (nothing by nothing) is free as well.
	 * Replacing something by null is delegated to {@link #getDeleteWeight(Object)}.
	 * Replacing null by something is delegated to {@link #getInsertWeight(Object)}.
	 * @see WeightCalculator#getReplaceWeight(Object, Object)
	 */
	public float getReplaceWeight(T seg1, T seg2) {
		if (seg1 == null && seg2 != null) {
			return getInsertWeight(seg2);
		} else if (seg1 != null && seg2 == null) {
			return getDeleteWeight(seg2);
		} else if ( seg1 == null && seg2 == null ) {
			return 0;
		}
		
		if (seg1.equals(seg2)) {
			return 0;
		} else {
			return 1;
		}
	}

}
