/*
Copyright (C) 2007-2009 Ramon Ziai
Copyright (C) 2007-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */


package de.linuxusers.levenshtein.editweight;

/**
 * Generic interface for implementing your own edit weights for the Levensthein algorithm.
 * The features of the type parameter <T> depend on the implementation.
 * 
 * @author Ramon Ziai
 * @author Niels Ott
 * @param  <T> the type of objects in a sequence that should be compared. 
 * @version $Id: WeightCalculator.java 36 2009-02-05 17:42:15Z niels $
 */
public interface WeightCalculator<T> {

	/**
	 * Returns the cost or weight for replacing the object in the first argument
	 * by the second argument.
	 * Either argument is allowed to be <code>null</code>. Implementers of
	 * the interface must take care of this.
	 * @param seg1 the element to replace.
	 * @param seg2 the replacement element.
	 * @return the cost for this operation.
	 */
	public float getReplaceWeight(T seg1, T seg2);
	
	/**
	 * Returns the cost or weight for inserting an object.
	 * The argument is allowed to be <code>null</code>. Implementers of
	 * the interface must take care of this.
	 * @param ins the object to insert.
	 * @return the cost for this operation.
	 */
	public float getInsertWeight(T ins);

	/**
	 * Returns the cost or weight for deleting an object.
	 * The argument is allowed to be <code>null</code>. Implementers of
	 * the interface must take care of this.
	 * @param del the object to delete.
	 * @return the cost for this operation.
	 */
	public float getDeleteWeight(T del);
	
}
